@@ -3,6 +3,7 @@ package dns
 import (
 	"context"
 	"fmt"
+	"os"
 	"reflect"
 	"time"
 
@@ -18,8 +19,10 @@ import (
 	gcpdns "github.com/openshift/cluster-ingress-operator/pkg/dns/gcp"
 	logf "github.com/openshift/cluster-ingress-operator/pkg/log"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
+	"github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
 	operatorutil "github.com/openshift/cluster-ingress-operator/pkg/util"
 	oputil "github.com/openshift/cluster-ingress-operator/pkg/util"
+	awsutil "github.com/openshift/cluster-ingress-operator/pkg/util/aws"
 	"github.com/openshift/cluster-ingress-operator/pkg/util/slice"
 
 	corev1 "k8s.io/api/core/v1"
@@ -52,13 +55,18 @@ const (
 	// operator's namespace that will hold the credentials that the operator
 	// will use to authenticate with the cloud API.
 	cloudCredentialsSecretName = "cloud-credentials"
+
+	// kubeCloudConfigName is the name of the kube cloud config ConfigMap
+	kubeCloudConfigName = "kube-cloud-config"
+	// cloudCABundleKey is the key in the kube cloud config ConfigMap where the custom CA bundle is located
+	cloudCABundleKey = "ca-bundle.pem"
 )
 
 var log = logf.Logger.WithName(controllerName)
 
 func New(mgr manager.Manager, config Config) (runtimecontroller.Controller, error) {
 	reconciler := &reconciler{
-		Config:   config,
+		config:   config,
 		client:   mgr.GetClient(),
 		cache:    mgr.GetCache(),
 		recorder: mgr.GetEventRecorderFor(controllerName),
@@ -67,7 +75,7 @@ func New(mgr manager.Manager, config Config) (runtimecontroller.Controller, erro
 	if err != nil {
 		return nil, err
 	}
-	if err := c.Watch(&source.Kind{Type: &iov1.DNSRecord{}}, &handler.EnqueueRequestForObject{}); err != nil {
+	if err := c.Watch(&source.Kind{Type: &iov1.DNSRecord{}}, &handler.EnqueueRequestForObject{}, predicate.GenerationChangedPredicate{}); err != nil {
 		return nil, err
 	}
 	if err := c.Watch(&source.Kind{Type: &configv1.DNS{}}, &handler.EnqueueRequestsFromMapFunc{ToRequests: handler.ToRequestsFunc(reconciler.ToDNSRecords)}); err != nil {
@@ -101,7 +109,7 @@ type Config struct {
 }
 
 type reconciler struct {
-	Config
+	config Config
 
 	client           client.Client
 	cache            cache.Cache
@@ -214,7 +222,7 @@ func (r *reconciler) createDNSProviderIfNeeded(dnsConfig *configv1.DNS) error {
 	creds := &corev1.Secret{}
 	switch platformStatus.Type {
 	case configv1.AWSPlatformType, configv1.AzurePlatformType, configv1.GCPPlatformType:
-		name := types.NamespacedName{Namespace: r.Config.Namespace, Name: cloudCredentialsSecretName}
+		name := types.NamespacedName{Namespace: r.config.Namespace, Name: cloudCredentialsSecretName}
 		if err := r.cache.Get(context.TODO(), name, creds); err != nil {
 			return fmt.Errorf("failed to get cloud credentials from secret %s: %v", name, err)
 		}
@@ -260,17 +268,34 @@ func (r *reconciler) publishRecordToZones(zones []configv1.DNSZone, record *iov1
 			LastTransitionTime: metav1.Now(),
 		}
 
-		if err := r.dnsProvider.Ensure(record, zone); err != nil {
-			log.Error(err, "failed to publish DNS record to zone", "record", record.Spec, "dnszone", zone)
-			condition.Status = string(operatorv1.ConditionTrue)
-			condition.Reason = "ProviderError"
-			condition.Message = fmt.Sprintf("The DNS provider failed to ensure the record: %v", err)
-			result.RequeueAfter = 30 * time.Second
+		if recordIsAlreadyPublishedToZone(record, &zone) {
+			log.Info("replacing DNS record", "record", record.Spec, "dnszone", zone)
+
+			if err := r.dnsProvider.Replace(record, zone); err != nil {
+				log.Error(err, "failed to replace DNS record in zone", "record", record.Spec, "dnszone", zone)
+				condition.Status = string(operatorv1.ConditionTrue)
+				condition.Reason = "ProviderError"
+				condition.Message = fmt.Sprintf("The DNS provider failed to replace the record: %v", err)
+				result.RequeueAfter = 30 * time.Second
+			} else {
+				log.Info("replaced DNS record in zone", "record", record.Spec, "dnszone", zone)
+				condition.Status = string(operatorv1.ConditionFalse)
+				condition.Reason = "ProviderSuccess"
+				condition.Message = "The DNS provider succeeded in replacing the record"
+			}
 		} else {
-			log.Info("published DNS record to zone", "record", record.Spec, "dnszone", zone)
-			condition.Status = string(operatorv1.ConditionFalse)
-			condition.Reason = "ProviderSuccess"
-			condition.Message = "The DNS provider succeeded in ensuring the record"
+			if err := r.dnsProvider.Ensure(record, zone); err != nil {
+				log.Error(err, "failed to publish DNS record to zone", "record", record.Spec, "dnszone", zone)
+				condition.Status = string(operatorv1.ConditionTrue)
+				condition.Reason = "ProviderError"
+				condition.Message = fmt.Sprintf("The DNS provider failed to ensure the record: %v", err)
+				result.RequeueAfter = 30 * time.Second
+			} else {
+				log.Info("published DNS record to zone", "record", record.Spec, "dnszone", zone)
+				condition.Status = string(operatorv1.ConditionFalse)
+				condition.Reason = "ProviderSuccess"
+				condition.Message = "The DNS provider succeeded in ensuring the record"
+			}
 		}
 		statuses = append(statuses, iov1.DNSZoneStatus{
 			DNSZone:    zone,
@@ -303,6 +328,11 @@ func (r *reconciler) delete(record *iov1.DNSRecord) error {
 	var errs []error
 	for i := range record.Status.Zones {
 		zone := record.Status.Zones[i].DNSZone
+		// If the record is currently not published in a zone,
+		// skip deleting it for that zone.
+		if !recordIsAlreadyPublishedToZone(record, &zone) {
+			continue
+		}
 		err := r.dnsProvider.Delete(record, zone)
 		if err != nil {
 			errs = append(errs, err)
@@ -400,7 +430,7 @@ func dnsZoneStatusSlicesEqual(a, b []iov1.DNSZoneStatus) bool {
 func (r *reconciler) ToDNSRecords(o handler.MapObject) []reconcile.Request {
 	var requests []reconcile.Request
 	records := &operatoringressv1.DNSRecordList{}
-	if err := r.cache.List(context.Background(), records, client.InNamespace(r.Namespace)); err != nil {
+	if err := r.cache.List(context.Background(), records, client.InNamespace(r.config.Namespace)); err != nil {
 		log.Error(err, "failed to list dnsrecords", "related", o.Meta.GetSelfLink())
 		return requests
 	}
@@ -433,15 +463,23 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 	}
 
 	var dnsProvider dns.Provider
-	userAgent := fmt.Sprintf("OpenShift/%s (ingress-operator)", r.Config.OperatorReleaseVersion)
+	userAgent := fmt.Sprintf("OpenShift/%s (ingress-operator)", r.config.OperatorReleaseVersion)
 
 	switch platformStatus.Type {
 	case configv1.AWSPlatformType:
 		cfg := awsdns.Config{
-			AccessID:  string(creds.Data["aws_access_key_id"]),
-			AccessKey: string(creds.Data["aws_secret_access_key"]),
-			Region:    platformStatus.AWS.Region,
+			Region: platformStatus.AWS.Region,
 		}
+
+		sharedCredsFile, err := awsutil.SharedCredentialsFileFromSecret(creds)
+		if err != nil {
+			return nil, fmt.Errorf("failed to create shared credentials file from Secret: %v", err)
+		}
+		// since at the end of this function the aws dns provider will be initialized with aws clients, the AWS SDK no
+		// longer needs access to the file and therefore it can be removed.
+		defer os.Remove(sharedCredsFile)
+		cfg.SharedCredentialFile = sharedCredsFile
+
 		if len(platformStatus.AWS.ServiceEndpoints) > 0 {
 			cfg.ServiceEndpoints = []awsdns.ServiceEndpoint{}
 			route53Found := false
@@ -484,7 +522,13 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 				}
 			}
 		}
-		provider, err := awsdns.NewProvider(cfg, r.OperatorReleaseVersion)
+
+		cfg.CustomCABundle, err = r.customCABundle()
+		if err != nil {
+			return nil, fmt.Errorf("failed to get the custom CA bundle: %w", err)
+		}
+
+		provider, err := awsdns.NewProvider(cfg, r.config.OperatorReleaseVersion)
 		if err != nil {
 			return nil, fmt.Errorf("failed to create AWS DNS manager: %v", err)
 		}
@@ -500,7 +544,7 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 			ClientSecret:   string(creds.Data["azure_client_secret"]),
 			TenantID:       string(creds.Data["azure_tenant_id"]),
 			SubscriptionID: string(creds.Data["azure_subscription_id"]),
-		}, r.Config.OperatorReleaseVersion)
+		}, r.config.OperatorReleaseVersion)
 		if err != nil {
 			return nil, fmt.Errorf("failed to create Azure DNS manager: %v", err)
 		}
@@ -520,3 +564,25 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 	}
 	return dnsProvider, nil
 }
+
+// customCABundle will get the custom CA bundle, if present, configured in the kube cloud config.
+func (r *reconciler) customCABundle() (string, error) {
+	cm := &corev1.ConfigMap{}
+	switch err := r.client.Get(
+		context.Background(),
+		client.ObjectKey{Namespace: controller.GlobalMachineSpecifiedConfigNamespace, Name: kubeCloudConfigName},
+		cm,
+	); {
+	case errors.IsNotFound(err):
+		// no cloud config ConfigMap, so no custom CA bundle
+		return "", nil
+	case err != nil:
+		return "", fmt.Errorf("failed to get kube-cloud-config ConfigMap: %w", err)
+	}
+	caBundle, ok := cm.Data[cloudCABundleKey]
+	if !ok {
+		// no "ca-bundle.pem" key in the ConfigMap, so no custom CA bundle
+		return "", nil
+	}
+	return caBundle, nil
+}
