@@ -56,7 +56,7 @@ var clock utilclock.Clock = utilclock.RealClock{}
 // and uses them to compute the operator status.
 func New(mgr manager.Manager, config Config) (controller.Controller, error) {
 	reconciler := &reconciler{
-		Config: config,
+		config: config,
 		client: mgr.GetClient(),
 		cache:  mgr.GetCache(),
 	}
@@ -80,7 +80,7 @@ type Config struct {
 // reconciler handles the actual status reconciliation logic in response to
 // events.
 type reconciler struct {
-	Config
+	config Config
 
 	client client.Client
 	cache  cache.Cache
@@ -91,7 +91,8 @@ type reconciler struct {
 func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, error) {
 	log.Info("Reconciling", "request", request)
 
-	nsManifest := manifests.RouterNamespace()
+	ingressNamespace := manifests.RouterNamespace().Name
+	canaryNamespace := manifests.CanaryNamespace().Name
 
 	co := &configv1.ClusterOperator{ObjectMeta: metav1.ObjectMeta{Name: operatorcontroller.IngressClusterOperatorName().Name}}
 	if err := r.client.Get(context.TODO(), operatorcontroller.IngressClusterOperatorName(), co); err != nil {
@@ -107,7 +108,7 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	}
 	oldStatus := co.Status.DeepCopy()
 
-	state, err := r.getOperatorState(nsManifest.Name)
+	state, err := r.getOperatorState(ingressNamespace, canaryNamespace)
 	if err != nil {
 		return reconcile.Result{}, fmt.Errorf("failed to get operator state: %v", err)
 	}
@@ -115,25 +116,32 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	related := []configv1.ObjectReference{
 		{
 			Resource: "namespaces",
-			Name:     "openshift-ingress-operator",
+			Name:     r.config.Namespace,
 		},
 		{
 			Group:     operatorv1.GroupName,
 			Resource:  "IngressController",
-			Namespace: r.Namespace,
+			Namespace: r.config.Namespace,
 		},
 		{
 			Group:     iov1.GroupVersion.Group,
 			Resource:  "DNSRecord",
-			Namespace: r.Namespace,
+			Namespace: r.config.Namespace,
 		},
 	}
-	if state.Namespace != nil {
+	if state.IngressNamespace != nil {
 		related = append(related, configv1.ObjectReference{
 			Resource: "namespaces",
-			Name:     state.Namespace.Name,
+			Name:     state.IngressNamespace.Name,
 		})
 	}
+	if state.CanaryNamespace != nil {
+		related = append(related, configv1.ObjectReference{
+			Resource: "namespaces",
+			Name:     state.CanaryNamespace.Name,
+		})
+	}
+
 	co.Status.RelatedObjects = related
 
 	allIngressesAvailable := checkAllIngressesAvailable(state.IngressControllers)
@@ -141,7 +149,7 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	co.Status.Versions = r.computeOperatorStatusVersions(oldStatus.Versions, allIngressesAvailable)
 
 	co.Status.Conditions = mergeConditions(co.Status.Conditions, computeOperatorAvailableCondition(allIngressesAvailable))
-	co.Status.Conditions = mergeConditions(co.Status.Conditions, computeOperatorProgressingCondition(allIngressesAvailable, oldStatus.Versions, co.Status.Versions, r.OperatorReleaseVersion, r.IngressControllerImage))
+	co.Status.Conditions = mergeConditions(co.Status.Conditions, computeOperatorProgressingCondition(allIngressesAvailable, oldStatus.Versions, co.Status.Versions, r.config.OperatorReleaseVersion, r.config.IngressControllerImage))
 	co.Status.Conditions = mergeConditions(co.Status.Conditions, computeOperatorDegradedCondition(state.IngressControllers))
 
 	if !operatorStatusesEqual(*oldStatus, co.Status) {
@@ -182,28 +190,38 @@ func initializeClusterOperator(co *configv1.ClusterOperator) {
 }
 
 type operatorState struct {
-	Namespace          *corev1.Namespace
+	IngressNamespace   *corev1.Namespace
+	CanaryNamespace    *corev1.Namespace
 	IngressControllers []operatorv1.IngressController
 	DNSRecords         []iov1.DNSRecord
 }
 
 // getOperatorState gets and returns the resources necessary to compute the
 // operator's current state.
-func (r *reconciler) getOperatorState(nsName string) (operatorState, error) {
+func (r *reconciler) getOperatorState(ingressNamespace, canaryNamespace string) (operatorState, error) {
 	state := operatorState{}
 
-	ns := &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: nsName}}
-	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: nsName}, ns); err != nil {
+	ns := &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: ingressNamespace}}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: ingressNamespace}, ns); err != nil {
 		if !errors.IsNotFound(err) {
-			return state, fmt.Errorf("failed to get namespace %q: %v", nsName, err)
+			return state, fmt.Errorf("failed to get namespace %q: %v", ingressNamespace, err)
 		}
 	} else {
-		state.Namespace = ns
+		state.IngressNamespace = ns
+	}
+
+	ns = &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: canaryNamespace}}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: canaryNamespace}, ns); err != nil {
+		if !errors.IsNotFound(err) {
+			return state, fmt.Errorf("failed to get namespace %q: %v", canaryNamespace, err)
+		}
+	} else {
+		state.CanaryNamespace = ns
 	}
 
 	ingressList := &operatorv1.IngressControllerList{}
-	if err := r.cache.List(context.TODO(), ingressList, client.InNamespace(r.Namespace)); err != nil {
-		return state, fmt.Errorf("failed to list ingresscontrollers in %q: %v", r.Namespace, err)
+	if err := r.cache.List(context.TODO(), ingressList, client.InNamespace(r.config.Namespace)); err != nil {
+		return state, fmt.Errorf("failed to list ingresscontrollers in %q: %v", r.config.Namespace, err)
 	} else {
 		state.IngressControllers = ingressList.Items
 	}
@@ -222,11 +240,11 @@ func (r *reconciler) computeOperatorStatusVersions(oldVersions []configv1.Operan
 	return []configv1.OperandVersion{
 		{
 			Name:    OperatorVersionName,
-			Version: r.OperatorReleaseVersion,
+			Version: r.config.OperatorReleaseVersion,
 		},
 		{
 			Name:    IngressControllerVersionName,
-			Version: r.IngressControllerImage,
+			Version: r.config.IngressControllerImage,
 		},
 	}
 }
